@@ -24,12 +24,17 @@ try:
         finance,
     )
 
+    # 本地导入
 except ImportError:
-    pass
+    try:
+        from jqdata import finance  # 云平台导入
+    except ImportError:
+        pass
 
 from xalpha.info import fundinfo, mfundinfo
 from xalpha.cons import rget, rpost, rget_json, rpost_json, yesterday, opendate
 from xalpha.provider import data_source
+from xalpha.exceptions import DataPossiblyWrong
 
 
 thismodule = sys.modules[__name__]
@@ -280,6 +285,16 @@ def get_fundshare_byjq(code, **kws):
 
 
 def get_historical_fromsp(code, start=None, end=None, **kws):
+    """
+    标普官网数据源
+
+    :param code:
+    :param start:
+    :param end:
+    :param kws:
+    :return:
+    """
+
     if code.startswith("SP"):
         code = code[2:]
     if len(code.split(".")) > 1:
@@ -310,10 +325,55 @@ selectedModule=PerformanceGraphView&selectedSubModule=Graph\
     return df
 
 
+def get_historical_frombb(code, start=None, end=None, **kws):
+    """
+    https://www.bloomberg.com/ 数据源, 试验性支持。
+    似乎有很严格的 IP 封禁措施, 且最新数据更新滞后，似乎难以支持 T-1 净值预测
+
+    :param code:
+    :param start:
+    :param end:
+    :param kws:
+    :return:
+    """
+    if code.startswith("BB-"):
+        code = code[3:]
+    # end_obj = dt.datetime.strptime(end, "%Y%m%d")
+    start_obj = dt.datetime.strptime(start, "%Y%m%d")
+    today_obj = dt.datetime.now()
+    fromnow = (today_obj - start_obj).days
+    if fromnow < 20:
+        years = "1_MONTH"
+    elif fromnow < 300:
+        years = "1_YEAR"
+    else:
+        years = "5_YEAR"
+    url = "https://www.bloomberg.com/markets2/api/history/{code}/PX_LAST?\
+timeframe={years}&period=daily&volumePeriod=daily".format(
+        years=years, code=code
+    )
+    r = rget_json(
+        url,
+        headers={
+            "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4) AppleWebKit/537.36 (KHTML, like Gecko)",
+            "referer": "https://www.bloomberg.com/quote/{code}".format(code=code),
+            "sec-fetch-dest": "empty",
+            "sec-fetch-mode": "cors",
+            "sec-fetch-site": "same-origin",
+            "accept": "*/*",
+        },
+    )
+    df = pd.DataFrame(r[0]["price"])
+    df["close"] = df["value"]
+    df["date"] = pd.to_datetime(df["dateTime"])
+    df = df[["date", "close"]]
+    return df
+
+
 def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, **kws):
     """
     universal fetcher for daily historical data of literally everything has a value in market.
-    数据来源包括天天基金，雪球，英为财情，外汇局官网
+    数据来源包括天天基金，雪球，英为财情，外汇局官网，聚宽，标普官网，bloomberg 等。
 
     :param code: str.
 
@@ -337,15 +397,18 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
 
             10. 形如 fs-SH501018 格式的数据，可以返回指定场内基金每日份额，需要 enable 聚宽数据源方可查看。
 
-            11. 形如 SP5475707.2 格式的数据，可以返回标普官网相关指数的日线数据，id 5475707 部分可以从相关指数 export 按钮获取的链接中得到，小数点后的部分代表保存的列数。参考链接：https://us.spindices.com/indices/equity/sp-global-oil-index
+            11. 形如 SP5475707.2 格式的数据，可以返回标普官网相关指数的日线数据（最近十年），id 5475707 部分可以从相关指数 export 按钮获取的链接中得到，小数点后的部分代表保存的列数。参考链接：https://us.spindices.com/indices/equity/sp-global-oil-index
+
+            12. 形如 BB-FGERBIU:ID 格式的数据，对应网页 https://www.bloomberg.com/quote/FGERBIU:ID，可以返回彭博的数据（最近五年）
 
     :param start: str. "20200101", "2020/01/01", "2020-01-01" are all legal. The starting date of daily data.
     :param end: str. format is the same as start. The ending date of daily data.
     :param prev: Optional[int], default 365. If start is not specified, start = end-prev.
-    :param _from: Optional[str]. can be one of "xueqiu", "zjj", "investing", "tiantianjijin". Only used for debug to
+    :param _from: Optional[str]. 一般用户不需设定该选项。can be one of "xueqiu", "zjj", "investing", "tiantianjijin". Only used for debug to
         enfore data source. For common use, _from can be chosed automatically based on code in the run time.
+    :param wrapper: bool. 一般用户不需设定该选项。
     :return: pd.Dataframe.
-        must include cols: date[pd.Timestampe]，close[float64]。
+        must include cols: date[pd.Timestamp]，close[float64]。
     """
     if not end:
         end_obj = today_obj()
@@ -372,7 +435,7 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
             _from = "xueqiu"
             code = code[2:]
         elif code.startswith("SP") and code[2:].split(".")[0].isdigit():
-            _from = "sp"
+            _from = "SP"
         elif len(code.split("-")) == 2 and len(code.split("-")[0]) <= 3:
             # peb-000807.XSHG
             _from = code.split("-")[0]
@@ -404,9 +467,12 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
     elif _from == "fs":
         df = get_fundshare_byjq(code, start=start, end=end)
 
-    elif _from == "sp":
+    elif _from == "SP":
         df = get_historical_fromsp(code, start=start, end=end)
 
+    elif _from == "BB":
+        df = get_historical_frombb(code, start=start, end=end)
+
     if wrapper or len(df) == 0:
         return df
     else:
@@ -424,6 +490,8 @@ def _float(n):
 
 
 def get_xueqiu_rt(code, token="a664afb60c7036c7947578ac1a5860c4cfb6b3b5"):
+    if code.startswith("HK"):
+        code = code[2:]
     url = "https://stock.xueqiu.com/v5/stock/quote.json?symbol={code}&extend=detail"
     r = rget_json(
         url.format(code=code),
@@ -486,28 +554,69 @@ def get_cninvesting_rt(suburl):
     }
 
 
-def get_rt(code, _from=None):
+def get_rt_from_sina(code):
+    if (
+        code.startswith("SH") or code.startswith("SZ") or code.startswith("HK")
+    ) and code[2:].isdigit():
+        tinycode = code[:2].lower() + code[2:]
+    else:  # 美股
+        tinycode = "gb_"
+        if code.startswith("."):
+            code = code[1:]
+        tinycode += code.lower()
+    r = rget("https://hq.sinajs.cn/list={tinycode}".format(tinycode=tinycode))
+    l = r.text.split("=")[1].split(",")
+    d = {}
+    d["name"] = l[0].strip('"')
+    if (
+        code.startswith("SH") or code.startswith("SZ") or code.startswith("HK")
+    ) and code[2:].isdigit():
+        d["current"] = float(l[3])
+        d["percent"] = round((float(l[3]) / float(l[2]) - 1) * 100, 2)
+        d["current_ext"] = None
+        if code.startswith("HK"):
+            d["currency"] = "HKD"
+        else:
+            d["currency"] = "CNY"
+    else:
+        d["currency"] = "USD"
+        d["current"] = float(l[1])
+        d["percent"] = float(l[2])
+        d["current_ext"] = None
+    return d
+
+
+def get_rt(code, _from=None, double_check=False, double_check_threhold=0.005):
     """
     universal fetcher for realtime price of literally everything.
 
     :param code: str. 规则同 :func:`get_daily`. 需要注意场外基金和外汇中间价是不支持实时行情的，因为其每日只有一个报价。对于 investing 的数据源，只支持网址格式代码。
     :param _from: Optional[str]. can be one of "xueqiu", "investing". Only used for debug to
         enfore data source. For common use, _from can be chosed automatically based on code in the run time.
+    :param double_check: Optional[bool], default False. 如果设为 True，只适用于 A 股，美股，港股实时行情，会通过至少两个不同的数据源交叉验证，确保正确。
+            适用于需要自动交易等情形，防止实时数据异常。
     :return: Dict[str, Any].
         包括 "name", "current", "percent" 三个必有项和 "current_ext"（盘后价格）, "currency" （计价货币）两个值可能为 ``None`` 的选项。
     """
     if not _from:
         if len(code.split("/")) > 1:
             _from = "investing"
-        elif code.startswith("HK") and code[2:].isdigit():
-            _from = "xueqiu"
-            code = code[2:]
-        else:
+        # elif code.startswith("HK") and code[2:].isdigit():
+        #     _from = "xueqiu"
+        else:  # 默认不启用新浪实时，只做双重验证
             _from = "xueqiu"
     if _from in ["cninvesting", "investing"]:
         return get_cninvesting_rt(code)
+    elif double_check:
+        r1 = get_xueqiu_rt(code, token=get_token())
+        r2 = get_rt_from_sina(code)
+        if abs(r1["current"] / r2["current"] - 1) > double_check_threhold:
+            raise DataPossiblyWrong("realtime data unmatch for %s" % code)
+        return r1
     elif _from in ["xueqiu", "xq", "snowball"]:
         return get_xueqiu_rt(code, token=get_token())
+    elif _from in ["sina", "sn", "xinlang"]:
+        return get_rt_from_sina(code)
 
 
 get_realtime = get_rt
@@ -518,12 +627,13 @@ _cached_data = {}
 
 def reset_cache():
     """
-    clear all cache of daily data, used for :func:`cached`, deprecated.
+    clear all cache of daily data in memory.
 
     :return: None.
     """
     global _cached_data
     _cached_data = {}
+    setattr(thismodule, "cached_dict", {})
 
 
 def cached(s):
@@ -646,24 +756,32 @@ def cachedio(**ioconf):
             else:
                 if backend == "csv":
                     key = key + ".csv"
-                elif backend == "memory":
-                    if not getattr(thismodule, "cached_dict", None):
-                        setattr(thismodule, "cached_dict", {})
+                if not getattr(thismodule, "cached_dict", None):
+                    setattr(thismodule, "cached_dict", {})
                 if refresh:
+                    is_changed = True
                     df0 = f(*args, **kws)
 
                 else:  # non refresh
                     try:
                         if backend == "csv":
-                            df0 = pd.read_csv(os.path.join(path, key))
+                            if key in getattr(thismodule, "cached_dict"):
+                                # 即使硬盘级别的缓存，也有内存层，加快读写速度
+                                df0 = getattr(thismodule, "cached_dict")[key]
+                            else:
+                                df0 = pd.read_csv(os.path.join(path, key))
                         elif backend == "sql":
-                            df0 = pd.read_sql(key, path)
+                            if key in getattr(thismodule, "cached_dict"):
+                                df0 = getattr(thismodule, "cached_dict")[key]
+                            else:
+                                df0 = pd.read_sql(key, path)
                         elif backend == "memory":
                             df0 = getattr(thismodule, "cached_dict")[key]
                         else:
                             raise ValueError("no %s option for backend" % backend)
                         df0[date] = pd.to_datetime(df0[date])
                         # 向前延拓
+                        is_changed = False
                         if df0.iloc[0][date] > start_obj:
                             kws["start"] = start_str
                             kws["end"] = (
@@ -673,6 +791,7 @@ def cachedio(**ioconf):
                             if len(df1) > 0:
                                 df1 = df1[df1["date"] <= kws["end"]]
                             if len(df1) > 0:
+                                is_changed = True
                                 df0 = df1.append(df0, ignore_index=True)
                         # 向后延拓
                         if df0.iloc[-1][date] < end_obj:
@@ -687,6 +806,7 @@ def cachedio(**ioconf):
                             if len(df2) > 0:
                                 df2 = df2[df2["date"] >= kws["start"]]
                             if len(df2) > 0:
+                                is_changed = True
                                 if len(df0[df0["date"] == df0.iloc[-1]["date"]]) == 1:
                                     df0 = df0.iloc[:-1]
                                 df0 = df0.append(df2, ignore_index=True)
@@ -698,16 +818,18 @@ def cachedio(**ioconf):
                             kws["end"] = (today_obj() - dt.timedelta(days=1)).strftime(
                                 "%Y-%m-%d"
                             )
+                        is_changed = True
                         df0 = f(*args, **kws)
 
-                if df0 is not None and len(df0) > 0:
+                if df0 is not None and len(df0) > 0 and is_changed:
                     if backend == "csv":
                         df0.to_csv(os.path.join(path, key), index=False)
                     elif backend == "sql":
                         df0.to_sql(key, con=path, if_exists="replace", index=False)
-                    elif backend == "memory":
-                        d = getattr(thismodule, "cached_dict")
-                        d[key] = df0
+                    # elif backend == "memory":
+                    # 总是刷新内存层，即使是硬盘缓存
+                    d = getattr(thismodule, "cached_dict")
+                    d[key] = df0
 
             if df0 is not None:
                 df0 = df0[df0["date"] <= end_str]
@@ -813,8 +935,8 @@ def get_peb(index, date=None, table=False):
     :param table: Optioanl[bool], default False. True 时返回整个计算的 DataFrame，用于 debug。
     :return: Dict[str, float]. 包含 pe 和 pb 值的字典。
     """
-    if len(index.split(".")) != 2:
-        index = _inverse_convert_code(index)
+    if len(index.split(".")) == 2:
+        index = _convert_code(index)
     middle = dt.datetime.strptime(
         date.replace("/", "").replace("-", ""), "%Y%m%d"
     ).replace(day=1)
@@ -863,197 +985,3 @@ def _inverse_convert_code(code):
         return code[2:] + ".XSHG"
     elif code.startswith("SZ"):
         return code[2:] + ".XSHE"
-
-
-class PEBHistory:
-    """
-    对于指数历史 PE PB 的封装类
-    """
-
-    indexs = {
-        "000016.XSHG": ("上证50", "2012-01-01"),
-        "000300.XSHG": ("沪深300", "2012-01-01"),
-        "000905.XSHG": ("中证500", "2012-01-01"),
-        "000922.XSHG": ("中证红利", "2012-01-01"),
-        "399006.XSHE": ("创业板指", "2012-01-01"),
-        "000992.XSHG": ("全指金融", "2012-01-01"),
-        "000991.XSHG": ("全指医药", "2012-01-01"),
-        "399932.XSHE": ("中证消费", "2012-01-01"),
-        "000831.XSHG": ("500低波", "2013-01-01"),
-        "000827.XSHG": ("中证环保", "2013-01-01"),
-        "000978.XSHG": ("医药100", "2012-01-01"),
-        "399324.XSHE": ("深证红利", "2012-01-01"),
-        "399971.XSHE": ("中证传媒", "2014-07-01"),
-        "000807.XSHG": ("食品饮料", "2013-01-01"),
-        "000931.XSHG": ("中证可选", "2012-01-01"),
-        "399812.XSHE": ("养老产业", "2016-01-01"),
-    }
-
-    # 聚宽数据源支持的指数列表： https://www.joinquant.com/indexData
-
-    def __init__(self, code, start=None, end=None):
-        """
-
-        :param code: str. 形式可以是 399971.XSHE 或者 SH000931
-        :param start: Optional[str]. %Y-%m-%d, 估值历史计算的起始日。
-        :param end: Dont use, only for debug
-        """
-        yesterday_str = (dt.datetime.now() - dt.timedelta(days=1)).strftime("%Y-%m-%d")
-        if len(code.split(".")) == 2:
-            self.code = code
-            self.scode = _convert_code(code)
-        else:
-            self.scode = code
-            self.code = _inverse_convert_code(self.scode)
-        if self.code in self.indexs:
-            self.name = self.indexs[self.code][0]
-            if not start:
-                start = self.indexs[self.code][1]
-        else:
-            try:
-                self.name = get_rt(self.scode)["name"]
-            except:
-                self.name = self.scode
-            if not start:
-                start = "2012-01-01"  # 可能会出问题，对应指数还未有数据
-        self.start = start
-        if not end:
-            end = yesterday_str
-        self.df = get_daily("peb-" + self.scode, start=self.start, end=end)
-        self.ratio = None
-        self.pep = [
-            round(i, 3) for i in np.percentile(self.df.pe, np.arange(0, 110, 10))
-        ]
-        self.pbp = [
-            round(i, 3) for i in np.percentile(self.df.pb, np.arange(0, 110, 10))
-        ]
-
-    def percentile(self):
-        """
-        打印 PE PB 的历史十分位对应值
-
-        :return:
-        """
-        print("PE 历史分位:\n")
-        print(*zip(np.arange(0, 110, 10), self.pep), sep="\n")
-        print("\nPB 历史分位:\n")
-        print(*zip(np.arange(0, 110, 10), self.pbp), sep="\n")
-
-    def v(self, y="pe"):
-        """
-        pe 或 pb 历史可视化
-
-        :param y: Optional[str]. "pe" (defualt) or "pb"
-        :return:
-        """
-        return self.df.plot(x="date", y=y)
-
-    def fluctuation(self):
-        if not self.ratio:
-            d = self.df.iloc[-1]["date"]
-            oprice = get_daily(code=self.scode, end=d.strftime("%Y%m%d"), prev=20).iloc[
-                -1
-            ]["close"]
-            nprice = get_rt(self.scode)["current"]
-            self.ratio = nprice / oprice
-        return self.ratio
-
-    def current(self, y="pe"):
-        """
-        返回实时的 pe 或 pb 绝对值估计。
-
-        :param y: Optional[str]. "pe" (defualt) or "pb"
-        :return: float.
-        """
-        return round(self.df.iloc[-1][y] * self.fluctuation(), 3)
-
-    def current_percentile(self, y="pe"):
-        """
-        返回实时的 pe 或 pb 历史百分位估计
-
-        :param y: Optional[str]. "pe" (defualt) or "pb"
-        :return: float.
-        """
-        df = self.df
-        d = len(df)
-        u = len(df[df[y] < self.current(y)])
-        return round(u / d * 100, 2)
-
-    def summary(self):
-        """
-        打印现在估值的全部分析信息。
-
-        :return:
-        """
-        print("指数%s估值情况\n" % self.name)
-        if dt.datetime.strptime(self.start, "%Y-%m-%d") > dt.datetime(2015, 1, 1):
-            print("(历史数据较少，仅供参考)\n")
-        #         self.percentile()
-        print(
-            "现在 PE 绝对值 %s, 相对分位 %s%%，距离最低点 %s %%\n"
-            % (
-                self.current("pe"),
-                self.current_percentile("pe"),
-                max(
-                    round(
-                        (self.current("pe") - self.pep[0]) / self.current("pe") * 100, 1
-                    ),
-                    0,
-                ),
-            )
-        )
-        print(
-            "现在 PB 绝对值 %s, 相对分位 %s%%，距离最低点 %s %%\n"
-            % (
-                self.current("pb"),
-                self.current_percentile("pb"),
-                max(
-                    round(
-                        (self.current("pb") - self.pbp[0]) / self.current("pb") * 100, 1
-                    ),
-                    0,
-                ),
-            )
-        )
-
-
-class Compare:
-    def __init__(self, *codes, start="20200101", end=yesterday()):
-        """
-
-        :param codes: Union[str, tuple], 格式与 :func:`_get_daily` 相同，若需要汇率转换，需要用 tuple，第二个元素形如 "USD"
-        :param start: %Y%m%d
-        :param end: %Y%m%d, default yesterday
-        """
-        totdf = pd.DataFrame()
-        codelist = []
-        for c in codes:
-            if isinstance(c, tuple):
-                code = c[0]
-                currency = c[1]
-            else:
-                code = c
-                currency = "CNY"  # 标的不做汇率调整
-            codelist.append(code)
-            df = get_daily(code, start=start, end=end)
-            df = df[df.date.isin(opendate)]
-            if currency != "CNY":
-                cdf = get_daily(currency + "/CNY", start=start, end=end)
-                cdf = cdf[cdf["date"].isin(opendate)]
-                df = df.merge(right=cdf, on="date", suffixes=("_x", "_y"))
-                df["close"] = df["close_x"] * df["close_y"]
-            df[code] = df["close"] / df.iloc[0].close
-            df = df.reset_index()
-            df = df[["date", code]]
-            if "date" not in totdf.columns:
-                totdf = df
-            else:
-                totdf = totdf.merge(on="date", right=df)
-        self.totdf = totdf
-        self.codes = codelist
-
-    def v(self):
-        return self.totdf.plot(x="date", y=self.codes)
-
-    def corr(self):
-        return self.totdf.iloc[:, 1:].pct_change().corr()
